#include "CheckSignature.h"

QUERY_INFO_PROCESS ZwQueryInformationProcess = NULL;

BOOLEAN
RequestCheckForSignatureAndPendIrp(
	PEPROCESS pEProcess,
	PIRP pIrp
	)
/*++
Routine Description:

    This routine use the pended Irps to get the WinVerifyTrust results

Arguments:

    PIRP	is the irp that requested hide and we are pending until usermode service clears us

Return Value:

    TRUE if the Check succedeed
	otherwise FALSE

--*/
{
	BOOLEAN status = FALSE;
	HANDLE hProcess = NULL;
	ULONG ImageNameSize = 0;
	UNICODE_STRING ZwQueryInformationProcessName = {0};
	UNICODE_STRING TranslatedImageName = {0};

	struct {
		UNICODE_STRING uImageName;
		WCHAR Buffer[1];
	} * pImageName = NULL;

	if(ZwQueryInformationProcess == NULL)
	{
		RtlInitUnicodeString(&ZwQueryInformationProcessName,L"ZwQueryInformationProcess");

		ZwQueryInformationProcess = (QUERY_INFO_PROCESS) MmGetSystemRoutineAddress(&ZwQueryInformationProcessName);

		if(ZwQueryInformationProcess == NULL) return(status);
	}

	hProcess = ZwCurrentProcess();

	if(hProcess != NULL)
	{

		if(ZwQueryInformationProcess(
				hProcess,
				ProcessImageFileName,
				NULL,
				0,
				&ImageNameSize
				) != STATUS_INFO_LENGTH_MISMATCH
			) return(status); //we use the function to retrieve the string size		

		pImageName = ExAllocatePoolWithTag(
			NonPagedPool,
			ImageNameSize + sizeof(UNICODE_STRING),
			'GMIe'
			);

		if(pImageName != NULL)
		{
			RtlZeroMemory(pImageName,ImageNameSize + sizeof(UNICODE_STRING));
			//
			//we init the UNICODE_STRING
			//
			//the ZwQueryInformationProcess always use UNICODE so we dont use TCHAR but WCHAR
			pImageName->uImageName.Buffer = pImageName->Buffer;
			pImageName->uImageName.Length = 0;
			pImageName->uImageName.MaximumLength = (USHORT)ImageNameSize - sizeof(UNICODE_STRING); //we need to translate from WCHAR to bytes

			if(NT_SUCCESS(ZwQueryInformationProcess(
							hProcess,
							ProcessImageFileName,
							pImageName,
							ImageNameSize + sizeof(UNICODE_STRING),
							&ImageNameSize
							))
			)
			{
				//
				//we got the image name
				//
				//we can use %S in this call because we are in the IRP_MJ_DEVICE_CONTROL dispatch
				//thus IRQL == PASSIVE_LEVEL
				DbgPrint("TestDriver - RequestCheckForSignatureAndPendIrp() - Image found: %S\n", pImageName->Buffer);
				//
				//we initialize the buffer to hold the translated imagename
				//
				TranslatedImageName.Buffer = 
					ExAllocatePoolWithTag(
						NonPagedPool,
						ImageNameSize,
						'GMIt'
						);

				RtlZeroMemory(TranslatedImageName.Buffer,ImageNameSize);

				TranslatedImageName.Length = 0;

				TranslatedImageName.MaximumLength = (USHORT)ImageNameSize;
				//
				//we call the TranslateImageName to get a Userspace filename
				//
				status = 
					TranslateImageName(
						&pImageName->uImageName,
						&TranslatedImageName
						);
				
				if(status == TRUE)
				{
					DbgPrint("TestDriver - CheckForSignature() - Image translated to: %S\n",TranslatedImageName.Buffer);
					//
					//We got a request to check for signature and we succesfully translated image name for user mode
					//
					status = 
						SendCheckForSignatureRequestToServiceAndPendIrp(
							&TranslatedImageName,
							pEProcess,
							pIrp
							);
				}
				else
					DbgPrint("TestDriver - CheckForSignature() - Error translating image name\n");

				ExFreePoolWithTag(TranslatedImageName.Buffer, 'GMIt');

			}

			ExFreePoolWithTag(pImageName,'GMIe'); //we free the previously allocated space

			pImageName = NULL;
		}
	}

	return(status);
};


BOOLEAN
TranslateImageName(
	PUNICODE_STRING pSource,
	PUNICODE_STRING pDestination
	)
/*++
Routine Description:

    This routine translate a kernel mode image name to user mode image name

Arguments:

    pSource		The source string to translate
	pDestination		Caller allocated (also buffer) UNICODE_STRING that receives the result

Return Value:

    TRUE if the Check succedeed
	otherwise FALSE

--*/
{
	BOOLEAN status = FALSE;
	UNICODE_STRING HarddiskVolumeBaseName = {0};
	UNICODE_STRING HarddiskVolumeName = {0};
	UNICODE_STRING PartFileName = {0};
	UNICODE_STRING TranslatedHardDiskVolumeName = {0};
	PWCHAR pHarddiskVolumeNameBuffer = NULL;
	PWCHAR pPartFileNameBuffer = NULL;
	SIZE_T HarddiskVolumeNameBufferSize = 0;
	SIZE_T PartFileNameBufferSize = 0;
	SIZE_T StringCompareResult = 0;
	SIZE_T i = 0;
	BOOLEAN SlashFound = FALSE;

	RtlInitUnicodeString(&HarddiskVolumeBaseName,L"\\Device\\HarddiskVolume");
	__try {
		//
		//we compare the source string to the base string
		//
		StringCompareResult = RtlCompareMemory(pSource->Buffer,HarddiskVolumeBaseName.Buffer,HarddiskVolumeBaseName.Length);

		if(StringCompareResult == HarddiskVolumeBaseName.Length)
		{	
			if(pSource->Length > HarddiskVolumeBaseName.Length)
			{
				for(i=HarddiskVolumeBaseName.Length;i < pSource->MaximumLength;i = i + sizeof(WCHAR))
				{
					PUCHAR SourceBufferPtr = (PUCHAR)pSource->Buffer;

					if(RtlCompareMemory(&SourceBufferPtr[i],L"\\",sizeof(WCHAR)) == sizeof(WCHAR)) 
					{
						SlashFound = TRUE;
						break;
					}
				}
				//
				//we found the slash that separates the filename
				//
				if(SlashFound)
				{
					HarddiskVolumeNameBufferSize = i + sizeof(WCHAR);
					PartFileNameBufferSize = pSource->Length - i +sizeof(WCHAR);

					pHarddiskVolumeNameBuffer = 
						ExAllocatePoolWithTag(
							NonPagedPool,
							HarddiskVolumeNameBufferSize,
							'BNVH'
							); //we allocate enough space to hold the hard disk name

					pPartFileNameBuffer =
						ExAllocatePoolWithTag(
							NonPagedPool,
							PartFileNameBufferSize,
							'BNFP'
							); //we allocate enough space to hold the file name part

					if(pHarddiskVolumeNameBuffer != NULL && pPartFileNameBuffer != NULL )
					{
						RtlZeroMemory(pHarddiskVolumeNameBuffer,HarddiskVolumeNameBufferSize);

						RtlCopyMemory(pHarddiskVolumeNameBuffer,pSource->Buffer,i);

						RtlZeroMemory(pPartFileNameBuffer,PartFileNameBufferSize);

						RtlCopyMemory(pPartFileNameBuffer,&pSource->Buffer[i / sizeof(WCHAR)],pSource->Length - i);

						DbgPrint("TestDriver - TranslateImageName() - Found hard disk: %S\n",pHarddiskVolumeNameBuffer);

						DbgPrint("TestDriver - TranslateImageNAme() - Found partial name: %S\n",pPartFileNameBuffer);

						RtlInitUnicodeString(
							&HarddiskVolumeName,
							pHarddiskVolumeNameBuffer
							);
						//
						//we translate the volume name to a DOS name
						//
						if(TranslateHarddiskName(&HarddiskVolumeName, &TranslatedHardDiskVolumeName))
						{
							DbgPrint("TestDriver - TranslateImageName() - Translated hard disk name: %S\n", TranslatedHardDiskVolumeName.Buffer);

							RtlInitUnicodeString(
								&PartFileName,
								pPartFileNameBuffer
								);

							RtlUnicodeStringCat(
								pDestination,
								&TranslatedHardDiskVolumeName
								); //we append the translated drive name

							RtlUnicodeStringCat(
								pDestination,
								&PartFileName
								); //we append the file name part

							status = TRUE;

							ExFreePool(TranslatedHardDiskVolumeName.Buffer);
						}

						ExFreePoolWithTag(pHarddiskVolumeNameBuffer,'BNVH');

						pHarddiskVolumeNameBuffer = NULL;

						ExFreePoolWithTag(pPartFileNameBuffer,'BNFP');

						pPartFileNameBuffer = NULL;
					}
					else
					{
						if(pPartFileNameBuffer != NULL) ExFreePoolWithTag(pPartFileNameBuffer,'BNFP');

						if(pHarddiskVolumeNameBuffer != NULL) ExFreePoolWithTag(pHarddiskVolumeNameBuffer,'BNVH');
					}
				}
			}	
		}
	} __except(EXCEPTION_EXECUTE_HANDLER) {

		status = FALSE;

		if(pHarddiskVolumeNameBuffer != NULL) ExFreePoolWithTag(pHarddiskVolumeNameBuffer,'BNVH');

		if(TranslatedHardDiskVolumeName.Buffer != NULL) ExFreePool(TranslatedHardDiskVolumeName.Buffer);
	}

	return(status);
}

BOOLEAN
TranslateHarddiskName(
	PUNICODE_STRING pSource,
	PUNICODE_STRING pDestination
	)
{
	BOOLEAN status = FALSE;
	NTSTATUS ntstatus = FALSE;
	OBJECT_ATTRIBUTES VolumeObjectAttributes = {0};
	IO_STATUS_BLOCK IoStatusBlock = {0};
	PFILE_OBJECT FileObjectPtr = NULL;
	PDEVICE_OBJECT VolumeObjectPtr = NULL;
	//PFILE_OBJECT FileObjectPtr = NULL;
	HANDLE hVolume = NULL;
	DWORD ParseContext = 0;

	__try {

		//
		//This one was hard to finally find, i tryed different suggestion found
		//googling around but no one worked!
		//In the end it was easyer than expected: FileObjectPtr->DeviceObject 
		//i spent 6 hours looking around
		//

		/*
		ntstatus = 
			ObReferenceObjectByName(
				pSource,
				64,
				0,
				0,
				NULL,
				KernelMode,
				0,
				&VolumeObjectPtr
				);

		DbgPrint("TestDriver - TranslateHarddiskName() - ObReferenceObjectByName returned: %0X\n");

		if(NT_SUCCESS(ntstatus))
		{
			ntstatus = 
					IoVolumeDeviceToDosName(
						VolumeObjectPtr,
						pDestination
						);

			if(NT_SUCCESS(ntstatus)) status = TRUE;

			ObDereferenceObject(VolumeObjectPtr);

			VolumeObjectPtr = NULL;
		}
		*/

		/*
		ntstatus = 
			IoGetDeviceObjectPointer(
				pSource,
				FILE_READ_DATA | FILE_SHARE_READ,
				&FileObjectPtr,
				&VolumeObjectPtr
				);

		DbgPrint("TestDriver - TranslateHarddiskName() - IoGetDeviceObjectPointer returned %0X\n",ntstatus);

		if(NT_SUCCESS(ntstatus))
		{
			ntstatus = 
				IoVolumeDeviceToDosName(
					VolumeObjectPtr,
					pDestination
					);

			if(NT_SUCCESS(ntstatus)) status = TRUE; //everything was ok

			ObDereferenceObject(FileObjectPtr);

			FileObjectPtr = NULL;
		}
		else
		{
			DbgPrint("TestDriver - TranslateHarddiskName() - Error translating hard disk name\n");
		}
		*/

		
		InitializeObjectAttributes(
			&VolumeObjectAttributes,
			pSource,
			OBJ_KERNEL_HANDLE | OBJ_CASE_INSENSITIVE,
			NULL,
			NULL
			);
		//
		//we open the volume
		//
		ntstatus = 
			ZwOpenFile(
				&hVolume,
				FILE_READ_DATA,
				&VolumeObjectAttributes,
				&IoStatusBlock,
				FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
				FILE_NON_DIRECTORY_FILE
				);
	
		if(NT_SUCCESS(ntstatus))
		{
			//
			//we get the reference to the volume Device Object
			//
			ntstatus = 
				ObReferenceObjectByHandle(
					hVolume,
					FILE_READ_DATA,
					*IoFileObjectType,
					KernelMode,
					(PVOID*)&FileObjectPtr,
					NULL
					);

			if(NT_SUCCESS(ntstatus))
			{
				//
				//We translate the name
				//

				//VolumeObjectPtr = IoGetRelatedDeviceObject(FileObjectPtr);

				ntstatus = 
					IoVolumeDeviceToDosName(
						FileObjectPtr->DeviceObject,
						pDestination
						);
	
				if(NT_SUCCESS(ntstatus)) status = TRUE; //if everything was ok
	
				ObDereferenceObject(FileObjectPtr);

				FileObjectPtr = NULL;
			}
	
			ZwClose(hVolume);

			hVolume = NULL;
			
		}
		
	} __except(EXCEPTION_EXECUTE_HANDLER) {

		status = FALSE;
		
		//if(FileObjectPtr != NULL) ObDereferenceObject(FileObjectPtr);

		if(hVolume != NULL) ZwClose(hVolume);

		if(FileObjectPtr != NULL) ObDereferenceObject(FileObjectPtr);

		//if(VolumeObjectPtr != NULL) ObDereferenceObject(VolumeObjectPtr);

	}
	
	return(status);
}


BOOLEAN
SendCheckForSignatureRequestToServiceAndPendIrp(
	PUNICODE_STRING pFileName,
	PEPROCESS pEProcess,
	PIRP pHideProcessIrp
	)
{
	BOOLEAN status = FALSE;
	PLIST_ENTRY pRequestIrpListEntry = NULL;
	PIRP pRequestIrp = NULL;
	KIRQL OldIrql = 0;
	PIO_STACK_LOCATION pRequestIrpCurrentIoStack = NULL;
	DWORD i = 0;
	//
	//we atomically remove the irp from the list.
	//we could have used the Interlocked list function but
	//because the cancel routines for the irp atomically verify that Flink
	//and Blink values of the Irp Tail.Overlay.ListEntry are
	//not NULL before cancelling it, we
	//need to atomically perform the set of Blink and Flink to NULL so
	//we need to perform those actions inside the spinlock guard
	//

	KeAcquireSpinLock(&WinVerifyPendingIrpListAcces,&OldIrql);

	if(!IsListEmpty(&WinVerifyPendingIrpListHead))
	{
		pRequestIrpListEntry = RemoveHeadList(&WinVerifyPendingIrpListHead);

		if(pRequestIrpListEntry != NULL)
		{
			pRequestIrpListEntry->Blink = NULL;
	
			pRequestIrpListEntry->Flink = NULL;
	
			KeReleaseSpinLock(&WinVerifyPendingIrpListAcces,OldIrql); //read comment on the other KeReleaseSpinLock
	
			pRequestIrp = CONTAINING_RECORD(pRequestIrpListEntry,IRP,Tail.Overlay.ListEntry);
	
			if(pRequestIrp != NULL)
			{
				//
				//now we can work on the Irp
				//
				pRequestIrpCurrentIoStack = IoGetCurrentIrpStackLocation(pRequestIrp);
	
				if(pRequestIrpCurrentIoStack->Parameters.DeviceIoControl.OutputBufferLength >= pFileName->Length + sizeof(WCHAR) + sizeof(DWORD))
				{
					//
					//we have space in the service DeviceIoControl buffer to hold the string and null char and the Handle
					//
					__try {
						//
						//we first fill some useful data and then pend the Hide Request Irp
						//
						pHideProcessIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_EPROCESS] = pEProcess;
	
						KeAcquireSpinLock(&HideProcessPendingIrpArrayAccess, &OldIrql);
						
						for(i=0;i<MAXIMUM_PENDING_HIDE_REQUEST;i++)
						{
							if(HideProcessPendingIrpArray[i] == NULL) break;
						}
	
						if(i<MAXIMUM_PENDING_HIDE_REQUEST)
						{
							//we found an handle
							pHideProcessIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE] = (PVOID)i;
	
							HideProcessPendingIrpArray[i] = pHideProcessIrp;
							//
							//we pend the hide process request irp
							//
							IoMarkIrpPending(pHideProcessIrp);
							
							IoSetCancelRoutine(pHideProcessIrp,HideProcessPendIrpCancel);
						}
						KeReleaseSpinLock(&HideProcessPendingIrpArrayAccess,OldIrql);
	
						DbgPrint("TestDriver - SendCheckForSignatureRequestToServiceAndPendIrp() - Created request handle: %d\n", i);
								
						RtlZeroMemory(
							pRequestIrp->AssociatedIrp.SystemBuffer,
							pRequestIrpCurrentIoStack->Parameters.DeviceIoControl.OutputBufferLength
							);
	
						RtlCopyMemory(
							pRequestIrp->AssociatedIrp.SystemBuffer,
							&i,
							sizeof(DWORD)
							);
	
						RtlCopyMemory(
							&((PUCHAR)(pRequestIrp->AssociatedIrp.SystemBuffer))[sizeof(DWORD)],
							pFileName->Buffer,
							pFileName->Length
							);
	
						//
						//and we complete one of the service Irp
						//
						pRequestIrp->IoStatus.Status = STATUS_SUCCESS;
	
						pRequestIrp->IoStatus.Information = sizeof(DWORD) + pFileName->Length + sizeof(WCHAR); //account also for terminating null char
	
						IoCompleteRequest(pRequestIrp,IO_NO_INCREMENT);
	
						status = TRUE;
	
					} __except(EXCEPTION_EXECUTE_HANDLER) {
	
						status = FALSE;
					}
	
				}
			}
		}	
		else
			//
			//we need to release the spinlock as quick as possible
			//that is why we have two calls in the two if branches
			//i could not wait all the TRUE branch to execute and call the
			//release after the if-else even though this is not some code that 
			//executes over and over thus holding the spin lock too long wouldnt 
			//cause performace issues. But lets stick with the <25us stated by MSDN
			//
			KeReleaseSpinLock(&WinVerifyPendingIrpListAcces,OldIrql);
	}
	else
		KeReleaseSpinLock(&WinVerifyPendingIrpListAcces,OldIrql);
	
	return(status);
}