#include "HideProcess.h"

PIRP HideProcessPendingIrpArray[MAXIMUM_PENDING_HIDE_REQUEST] = {0};

KSPIN_LOCK HideProcessPendingIrpArrayAccess = {0};

BOOLEAN
ConfirmHideProcess(
	PIRP pIrp
	)
{
	BOOLEAN status = FALSE;
	PIO_STACK_LOCATION  pCurrentIrpStack = NULL;
	DWORD * pParameters = NULL;
	DWORD PendingHideRequestIrpHandle = 0;
	DWORD PendingHideRequestIrpResponse = 0;
	PIRP pPendingHideRequestIrp = NULL;
	KIRQL OldIrql = 0;

	pCurrentIrpStack = IoGetCurrentIrpStackLocation(pIrp);

	DbgPrint("TestDriver - ConfirmHideProcess() - Received process confirmation\n");

	if(pCurrentIrpStack->Parameters.DeviceIoControl.InputBufferLength >= sizeof(DWORD) * 2)
	{
		//
		//There is data
		//
		pParameters = (DWORD*)pIrp->AssociatedIrp.SystemBuffer;

		PendingHideRequestIrpHandle = pParameters[HANDLE_ITEM];

		PendingHideRequestIrpResponse = pParameters[RESPONSE_ITEM];

		DbgPrint(
			"\tfor handle: %d\n\twith answer : %d\n",
			PendingHideRequestIrpHandle,
			PendingHideRequestIrpResponse
			);

		if(PendingHideRequestIrpHandle < MAXIMUM_PENDING_HIDE_REQUEST)
		{
			//
			//we have a possibly valid handle, lets get it
			//
			KeAcquireSpinLock(&HideProcessPendingIrpArrayAccess,&OldIrql);
			if(HideProcessPendingIrpArray[PendingHideRequestIrpHandle] != NULL)
			{
				pPendingHideRequestIrp = HideProcessPendingIrpArray[PendingHideRequestIrpHandle];

				HideProcessPendingIrpArray[PendingHideRequestIrpHandle] = NULL;
			}
			KeReleaseSpinLock(&HideProcessPendingIrpArrayAccess, OldIrql);
			//
			//Double check
			//
			if(
				(DWORD)(pPendingHideRequestIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE]) == 
				PendingHideRequestIrpHandle
				)
			{
				if(PendingHideRequestIrpResponse == 1)
				{
					//
					//hide the process
					//
					if(ExecuteHideProcess((PEPROCESS)(pPendingHideRequestIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_EPROCESS])))
						pPendingHideRequestIrp->IoStatus.Status = STATUS_SUCCESS;
					else
						pPendingHideRequestIrp->IoStatus.Status = STATUS_NOT_SUPPORTED;

					status = TRUE;
				}
				else
				{
					pPendingHideRequestIrp->IoStatus.Status = STATUS_NOT_SUPPORTED;
				}

				pPendingHideRequestIrp->IoStatus.Information = 0;

				IoCompleteRequest(pPendingHideRequestIrp,IO_NO_INCREMENT);
			}
			else
			{
				pPendingHideRequestIrp->IoStatus.Status = STATUS_NOT_SUPPORTED;

				pPendingHideRequestIrp->IoStatus.Information = 0;

				IoCompleteRequest(pPendingHideRequestIrp,IO_NO_INCREMENT);
			}
		}
	}
	
	return(status);
}

BOOLEAN
RequestHideProcess(
	PEPROCESS pEProcess,
	PIRP pIrp
	)
{ 
	RTL_OSVERSIONINFOW OSInformation = {0};
	PLIST_ENTRY pListEntry = NULL;
	PEPROCESS pNextProcess = NULL;
	BOOLEAN status = TRUE;
	NTSTATUS ntstatus = STATUS_SUCCESS;

	OSInformation.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW);

	ntstatus = RtlGetVersion(&OSInformation);

	if(NT_SUCCESS(ntstatus)) 
		DbgPrint("TestDriver - HideProcess() - OS Version INFO\n\tMajor Version: %lu\n\tMinor Version: %lu\n\tBuild Number: %lu\n\tPlatform ID: %lu\n\tService Pack: %.128S\n",
			OSInformation.dwMajorVersion,
			OSInformation.dwMinorVersion,
			OSInformation.dwBuildNumber,
			OSInformation.dwPlatformId,
			OSInformation.szCSDVersion
			);
	//
	//This debug print was for checking OS version as EPROCESS depends on that
	//if you want to make this driver work with multiple Windows versions that may have different
	//EPROCESS definition you need to expand EPROCESS definition (inside undocumented.h, 
	//derived from ntifs.h @ http://www.acc.umu.se/~bosse/) depending on the various buildnumber
	//and then handle the correct one depending on OSInformation.dwBuildNumber
	//in this case im playing with OS build 2600 (XP)
	//

	status = 
		RequestCheckForSignatureAndPendIrp(
			pEProcess, 
			pIrp
			);
	

	/*

	__try {

		pNextProcess = pEProcess;

		pListEntry = &pEProcess->ActiveProcessLinks; //we get our process linked list

		DbgPrint("TestDriver - HideProcess() - Enumerating found processes\n");
		//
		//we enumerate the processes, just for fun
		//
		do {
			DbgPrint("\t%.16s\n",
				&pNextProcess->ImageFileName[0]
				);

			pNextProcess = GetEProcessPtrFromListEntryPtr(pListEntry->Flink); //we get next process from the forward link

			pListEntry = pListEntry->Flink;

			if(pNextProcess == pEProcess) DbgPrint("TestDriver - HideProcess() - All EPROCESS list cycled\n");

		} while (!(pNextProcess == pEProcess || pNextProcess == NULL));
		//
		//then we unlink our EPROCESS from the list
		//
		pEProcess->ActiveProcessLinks.Blink->Flink = pEProcess->ActiveProcessLinks.Flink;
		pEProcess->ActiveProcessLinks.Flink->Blink = pEProcess->ActiveProcessLinks.Blink;

	} __except(EXCEPTION_EXECUTE_HANDLER) {

		status = FALSE;
	}
	*/

	return(status);
}

BOOLEAN
ExecuteHideProcess(
	PEPROCESS pEProcess
	)
{
	BOOLEAN status = FALSE;
	PEPROCESS pNextProcess = NULL;
	PLIST_ENTRY pListEntry = NULL;

	DbgPrint("TestDriver - ExecuteHideProcess() - Executing hide process\n");

	__try {

		pNextProcess = pEProcess;

		pListEntry = &pEProcess->ActiveProcessLinks; //we get our process linked list

		DbgPrint("TestDriver - ExecuteHideProcess() - Enumerating found processes\n");
		//
		//we enumerate the processes, just for fun
		//
		do {
			DbgPrint("\t%.16s\n",
				&pNextProcess->ImageFileName[0]
				);

			pNextProcess = GetEProcessPtrFromListEntryPtr(pListEntry->Flink); //we get next process from the forward link

			pListEntry = pListEntry->Flink;

			if(pNextProcess == pEProcess) DbgPrint("TestDriver - ExecuteHideProcess() - All EPROCESS list cycled\n");

		} while (!(pNextProcess == pEProcess || pNextProcess == NULL));
		//
		//then we unlink our EPROCESS from the list
		//
		pEProcess->ActiveProcessLinks.Blink->Flink = pEProcess->ActiveProcessLinks.Flink;
		pEProcess->ActiveProcessLinks.Flink->Blink = pEProcess->ActiveProcessLinks.Blink;

		status = TRUE;

	} __except(EXCEPTION_EXECUTE_HANDLER) {

		status = FALSE;
	}

	return(status);
}


void
HideProcessPendIrpCancel(
	PDEVICE_OBJECT pDeviceObject,
	PIRP pIrp
	)
{
	KIRQL OldIrql = 0;
	PIRP pHideProcessPendingIrp = NULL;

	DbgPrint("TestDriver - HideProcessPendIrpCancel() - Running cancel routine\n");

	IoReleaseCancelSpinLock(pIrp->CancelIrql); //we release the cancel spinlock as per MSDN
	//
	//we acquire the pendinng hide process requests array access
	//
	KeAcquireSpinLock(&HideProcessPendingIrpArrayAccess,&OldIrql);

	if((DWORD)pIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE] < MAXIMUM_PENDING_HIDE_REQUEST)
	{
		if(HideProcessPendingIrpArray[(DWORD)pIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE]] != NULL)
		{
			pHideProcessPendingIrp = 
				HideProcessPendingIrpArray[
					(DWORD)pIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE]
				];

			HideProcessPendingIrpArray[
				(DWORD)pIrp->Tail.Overlay.DriverContext[IRP_DRIVER_CONTEXT_HANDLE]
				] = NULL;
			KeReleaseSpinLock(&HideProcessPendingIrpArrayAccess,OldIrql);

			pHideProcessPendingIrp->IoStatus.Status = STATUS_CANCELLED;

			pIrp->IoStatus.Information = 0;

			IoCompleteRequest(pIrp, IO_NO_INCREMENT);
		}
		else
			KeReleaseSpinLock(&HideProcessPendingIrpArrayAccess,OldIrql);
		//
		//if we are here there is no Irp in the queue, 
		//maybe a concurrence with the code that completes the irp
		//
	}
	else
		KeReleaseSpinLock(&HideProcessPendingIrpArrayAccess,OldIrql);
	//
	//if we are here this irp is not part of our queue or something happened
	//
}


PEPROCESS 
GetEProcessPtrFromListEntryPtr(
	PLIST_ENTRY pListEntry
	)
/*++
Routine Description:

    This routine calculate the EPROCESS ptr starting from 
	the LIST_ENTRY ptr known as EPROCESS.ActiveProcessLinks

Arguments:

    pListEntry - The ptr to an EPROCESS.ActiveProcessLinks

Return Value:

    PEPROCESS if no violation error, 
	else NULL

--*/
{
	PEPROCESS pEProcess = NULL;

	__try{
		//
		//we offset using the CONTAINING_RECORD macro
		//
		pEProcess = CONTAINING_RECORD(pListEntry,EPROCESS,ActiveProcessLinks);

	} __except(EXCEPTION_EXECUTE_HANDLER) {

		pEProcess = NULL;
	}

	return(pEProcess);
}

