#include "CThread.h"

CThread::CThread() : StackSize(0), pThreadFunction(0),CreationFlags(0), ThreadId(0), Object(NULL) 
{
	ZeroMemory(&ThreadAttributes,sizeof(SECURITY_ATTRIBUTES)); 
	
	OutputDebugString(_T("CThread::CThread() - Created CThread Object \n"));
}

CThread::~CThread()
{	
	if (!IsNull())
	{
		Terminate();
	}
	OutputDebugString(_T("CThread::~CThread() - Destroyed CThread Object\n"));
}

BOOL CThread::IsNull(void)
{
	ThreadSync.Enter();

	if (Object == NULL) 
	{
		ThreadSync.Leave();

		return TRUE;		
	}
	ThreadSync.Leave();

	return FALSE;
	
	
}

void CThread::SetObject(HANDLE _Object) 
{ 
	ThreadSync.Enter(); 
	
	Object = _Object; 

	ThreadSync.Leave();
}

BOOL CThread::IsFunctionSet(void)
{
	if (pThreadFunction == 0) return FALSE;
	else return TRUE;
}

BOOL CThread::Terminate(void)
{
	return Terminate(0);
}

BOOL CThread::Terminate(DWORD ExitCode)
{
	if (!IsNull())
	{
		ThreadSync.Enter();

		if (TerminateThread(InternalGetObject(),ExitCode) == 0)
		{
			OutputDebugString(_T("CThread::terminate() - TerminateThread() Error terminating thread \n"));
			
			ThreadSync.Leave();

			return FALSE;
		}
		else
		{
			Object = NULL;
			
			OutputDebugString(_T("CThread::terminate() - TerminateThread() SUCCESSFUL \n"));

			ThreadSync.Leave();
			
			return TRUE;
		}
	}
	
	return FALSE;
}

DWORD CThread::Wait(void)
{
	return WaitTimeout(INFINITE);
}

HANDLE CThread::InternalGetObject(void)
{
	return Object;
}

HANDLE CThread::GetObject(void)
{ 
	HANDLE t_Object;

	ThreadSync.Enter();

	t_Object = InternalGetObject();

	ThreadSync.Leave();

	return t_Object; 
};

DWORD CThread::WaitTimeout(DWORD TimeOut)
{
	DWORD t_result;

	if (!IsNull())
	{
		//ThreadSync.Enter();

		t_result = WaitForSingleObject(GetObject(),TimeOut);

		//ThreadSync.Leave();

		return t_result;
	}

	OutputDebugString(_T("CThread::wait() - WaitForSingleObject() Error no thread to wait \n"));
	
	return WAIT_FAILED;
}

BOOL CThread::Suspend(void)
{
	if (!IsNull())
	{
		ThreadSync.Enter();

		if (SuspendThread(InternalGetObject()) == (DWORD)-1)
		{
			OutputDebugString(_T("CThread::suspend() - SuspendThread() Error suspending thread \n"));
			
			ThreadSync.Leave();

			return FALSE;
		}
		else
		{
			OutputDebugString(_T("CThread::suspend() - SuspendThread() SUCCESSFUL \n"));
			
			ThreadSync.Leave();

			return TRUE;
		}
	}
	
	OutputDebugString(_T("CThread::suspend() - SuspendThread() Error no thread to suspend \n"));
	
	return FALSE;
}

BOOL CThread::Resume(void)
{

	if (!IsNull())
	{
		ThreadSync.Enter();

		if (ResumeThread(Object) == (DWORD)-1)
		{	
			OutputDebugString(_T("CThread::resume() - ResumeThread() Error suspending thread \n"));

			ThreadSync.Leave();

			return FALSE;
		}
		else
		{
			OutputDebugString(_T("CThread::resume() - ResumeThread() SUCESSFUL \n"));

			ThreadSync.Leave();

			return TRUE;
		}
	}

	OutputDebugString(_T("CThread::resume() - ResumeThread() Error no thread to resume \n"));

	return FALSE;
}

BOOL CThread::Start(void)
{
	if (IsFunctionSet())
	{
		//we start the thread pointing to our wrap function
		//we start the thread suspended to give time to SetObject
		//to set the created handle before the thread exit, to prevent any
		//concurrent situation (i.e. that the thread is executed and ends before 
		//SetObject could set the handle, so the wrapper function set
		//handle to null before the SetObject below here, causing the 
		//handle to be set even if the thread is already dead with all the consequences)
		SetObject(CreateThread(&ThreadAttributes,StackSize,(LPTHREAD_START_ROUTINE)CThread::WrapThreadProc,(LPVOID)this,CreationFlags | CREATE_SUSPENDED,&ThreadId));
	
		if (IsNull())
		{
			OutputDebugString(_T("CThread::start() - CreateThread() Error creating new thread \n"));
		
			return FALSE;
		}
	
		OutputDebugString(_T("CThread::start() - CreateThread() SUCCESSFUL \n"));

		Resume();
	
		return TRUE;
	}
	
	return FALSE;
}

//wrapper for the thread function
DWORD WINAPI CThread::WrapThreadProc(LPVOID _pParameter)
{
	DWORD ret;
	
	CThread* my_thread_object = (CThread*)_pParameter;
	
	//we run the real threadproc 
	ret = (*(my_thread_object->GetThreadProc()))(my_thread_object->GetThreadpParam());
	
	//the thread is finished, we set handle to null
	my_thread_object->SetObject(NULL);
			
	return ret;
}


