#include "WinTrustProxySvc.h"


SERVICE_STATUS ServiceStatus = {0};
SERVICE_STATUS_HANDLE ServiceStatusHandle = NULL;
CCriticalSection ServiceStatusExclusiveAccess;
CEvent ServiceStopEvent;


VOID 
WINAPI
WinTrustProxySvc(
  DWORD dwArgc,
  LPTSTR *lpszArgv
)
{
	CThread * pDriverControlThread = NULL;

	OutputDebugString(_T("Entering WinTrustProxySvc\n\n"));

	ServiceStatusHandle = RegisterServiceCtrlHandler(SERVICE_NAME,WinTrustProxySvcCtrlHandler);

	if(!ServiceStatusHandle)
	{
		OutputDebugString(_T("WinTrustProxySvc - Error with RegisterServiceCtrlHandler\n\n"));
		return;
	}

	ServiceStatusExclusiveAccess.Enter();

	ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;

	ServiceStatus.dwServiceSpecificExitCode = 0;

	ServiceStatusExclusiveAccess.Leave();

	ReportSvcStatus(SERVICE_START_PENDING, NO_ERROR, 3000);

	if(ServiceStopEvent.Create(TRUE,FALSE,NULL) == FALSE) 
	{
		OutputDebugString(_T("WinTrustProxySvc - Cannot create ServiceStopEvent\n\n"));

		ReportSvcStatus(SERVICE_STOPPED, NO_ERROR, 0);

		return;
	}

	ReportSvcStatus(SERVICE_RUNNING, NO_ERROR, 0);

	while(1)
	{
		OutputDebugString(_T("WinTrustProxySvc - Working\n\n"));

		pDriverControlThread = new CThread();

		pDriverControlThread->SetThreadpParam(ServiceStopEvent.GetObject());

		pDriverControlThread->SetThreadProc(DriverControlThread);

		pDriverControlThread->Start();

		ServiceStopEvent.Wait(); //signal a shutdown

		pDriverControlThread->Wait(); //we wait for the driver control thread to stop

		OutputDebugString(_T("WinTrustProxySvc - Work finished\n\n"));

		ReportSvcStatus(SERVICE_STOPPED, NO_ERROR, 0);

		return;
	}

	return;
}

VOID 
WINAPI 
WinTrustProxySvcCtrlHandler( 
	DWORD CtrlId
	)
{
	switch(CtrlId)
	{
	case SERVICE_CONTROL_STOP:

		OutputDebugString(_T("WinTrustProxySvcCtrlHandler - Requested SERVICE_CONTROL_STOP\n\n"));

		ReportSvcStatus(SERVICE_STOP_PENDING, NO_ERROR, 0);

		ServiceStopEvent.SetEvent();

		ReportSvcStatus(ServiceStatus.dwCurrentState, NO_ERROR, 0);

	default:
		break;
	}

	return;
}


VOID
ReportSvcStatus(
	DWORD dwCurrentState,
    DWORD dwWin32ExitCode,
    DWORD dwWaitHint
	)
{
    static DWORD dwCheckPoint = 1;

    // Fill in the SERVICE_STATUS structure.
	ServiceStatusExclusiveAccess.Enter();

    ServiceStatus.dwCurrentState = dwCurrentState;
    ServiceStatus.dwWin32ExitCode = dwWin32ExitCode;
    ServiceStatus.dwWaitHint = dwWaitHint;

    if (dwCurrentState == SERVICE_START_PENDING)
        ServiceStatus.dwControlsAccepted = 0;
    else ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;

    if ( (dwCurrentState == SERVICE_RUNNING) ||
           (dwCurrentState == SERVICE_STOPPED) )
        ServiceStatus.dwCheckPoint = 0;
    else ServiceStatus.dwCheckPoint = dwCheckPoint++;

    // Report the status of the service to the SCM.
    SetServiceStatus( ServiceStatusHandle, &ServiceStatus );

	ServiceStatusExclusiveAccess.Leave();
}

DWORD
WINAPI
DriverControlThread (
	LPVOID lpParameter
	)
{
	DWORD status = ERROR_SUCCESS;
	HANDLE hProcHiderDevice = INVALID_HANDLE_VALUE;
	TCHAR * pDebugBuffer = NULL;
	TCHAR * pDeviceOutputBuffer = NULL;
	WCHAR * pFileName = NULL;
	DWORD DeviceOutputCount = 0;
	OVERLAPPED OverlapResult = {0};
	HANDLE ShutdownAndIOEventArray[2] = {0};
	DWORD WaitResult = 0;
	DWORD LastError = NO_ERROR;
	BOOLEAN ShutDownRequested = FALSE;
	BOOLEAN DeviceIoControlResult = FALSE;
	BOOLEAN GetOverlappedResultResult = FALSE;
	BOOLEAN UnknownError = FALSE;
	BOOLEAN FileSigned = FALSE;
	BOOLEAN GotData = FALSE;
	DWORD i = 0;
	DWORD ProcHiderDeviceResponseArray[2] = {0};
	//
	//This part was modified from initial design as CancelSynchronousIo/CancelIoEx 
	//is only supported by Windows Vista+. So, because in 2k/XP we cant cancel Synchronous 
	//outstanding IO from the WinTrustProxySvc thread we switch to Asynchronous IO
	//using FILE_FLAG_OVERLAPPED in the Createfile and an OVERLAPPED structure in DeviceIoControl
	//and need to use a multiple object Wait on the IO and on the Shutdown events and eventually
	//cancel the IO from within the thread that started the IO if the shutdown event is triggered.
	//	
	ShutdownAndIOEventArray[SHUTDOWN_EVENT] = (HANDLE)lpParameter;

	pDebugBuffer = (TCHAR*)malloc(sizeof(TCHAR)*1024); //we allocate 1024 char (not bytes)

	if(pDebugBuffer == NULL) 
	{
		status = ERROR_NOT_ENOUGH_MEMORY;

		return(status);
	}

	pDeviceOutputBuffer = (TCHAR*)malloc(sizeof(TCHAR)*1024); //we allocate 1024 char (not bytes)

	if(pDeviceOutputBuffer == NULL)
	{
		status = ERROR_NOT_ENOUGH_MEMORY;

		free(pDebugBuffer);

		return(status);
	}

	OutputDebugString(_T("DriverControlThread - Started Driver Control Thread\n"));

	if(WaitForSingleObject((HANDLE)lpParameter,0) == WAIT_OBJECT_0) ShutDownRequested = TRUE;

	do {
		UnknownError = FALSE;

		GotData = FALSE;

		OverlapResult.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

		ShutdownAndIOEventArray[IO_EVENT] = OverlapResult.hEvent;

		if(OverlapResult.hEvent == NULL)
		{
			UnknownError = TRUE;
		}

		hProcHiderDevice = CreateFile(
			DEVICE_NAME,
			FILE_ALL_ACCESS,
			FILE_SHARE_READ | FILE_SHARE_READ,
			NULL,
			OPEN_EXISTING,
			FILE_FLAG_OVERLAPPED, //we use Asynchronous I/O
			NULL
			);
		
		while(hProcHiderDevice != INVALID_HANDLE_VALUE && !ShutDownRequested && !UnknownError)
		{
			DeviceIoControlResult = 
				DeviceIoControl(
					hProcHiderDevice,
					IOCTL_TESTDRIVER_PENDING_USERMODE_REQUEST,
					NULL,
					0,
					pDeviceOutputBuffer,
					sizeof(TCHAR)*1024, //number of bytes
					&DeviceOutputCount,
					&OverlapResult
					);

			LastError = GetLastError();
			//
			//We had an error, check for PENDING I/O
			//
			if(DeviceIoControlResult == 0)
			{
				switch(LastError)
				{
				case ERROR_IO_PENDING:
				case ERROR_IO_INCOMPLETE:
					WaitResult = WaitForMultipleObjects(
						2,
						ShutdownAndIOEventArray,
						FALSE,
						INFINITE
						);

					switch(WaitResult)
					{
					case SHUTDOWN_EVENT:
						//we were signaled to shutdown so cancel outstanding I/O
						OutputDebugString(_T("DriverControlThread - Shutdown received\n"));
							
						ShutDownRequested = TRUE;
			
						CancelIo(hProcHiderDevice);
						//
						//This GetOverlappedResult should return ERROR_OPERATION_ABORTED
						//Should prevent this thread to cancel the Event while the OVERLAPPED 
						//operation is still in the I/O Manager by waiting on it
						//
						GetOverlappedResult(
							hProcHiderDevice,
							&OverlapResult,
							&DeviceOutputCount,
							TRUE
							);
					case IO_EVENT:
						//
						//we were signaled that I/O completed	
						//
						GetOverlappedResultResult = 
							GetOverlappedResult(
								hProcHiderDevice,
								&OverlapResult,
								&DeviceOutputCount,
								TRUE
								);
						if(GetOverlappedResultResult == 0)
						{
							UnknownError = TRUE;
						}
						else
						{
							GotData = TRUE;
						}
						break;
					} //switch(WaitResult)

					break;
				default:
					UnknownError = TRUE;

					break;
				} //switch(LastError)
				
			} //if(DeviceIoControlResult == 0)
			else
			{
				//
				//DeviceIoControl already returned data
				//
				GotData = TRUE;				
			}

			if(!UnknownError && GotData)
			{
				//
				//we got data, lets handle it
				//
				pFileName = (PWCHAR)((PCHAR)pDeviceOutputBuffer + sizeof(DWORD)); //we offset to get filename
	
				if(DeviceOutputCount > 0)
				{
					StringCchPrintf(
						pDebugBuffer,
						1024,
						_T("DriverControlThread - (IO_EVENT) GetOVerlappedResult returned %d bytes\n\tHandle: %d\n"),
						DeviceOutputCount,
						(DWORD*)pDeviceOutputBuffer[0]
						);
					
					OutputDebugString(pDebugBuffer);


					//
					//we strictly use a wide char debug string because the name from driver is 
					//returned as UNICODE to append the filename for debug
					//we could have used a strictly ANSI one and used %S instead of %s and the ANSI
					//version of OutputDebugString
					//
					StringCbPrintfW(
						pDebugBuffer,
						1024 * sizeof(TCHAR), //to have no problem if i am using ANSI or UNICODE i use a byte counted function instead of a char counted
						L"\tFilename: %s\n",
						pFileName
						);

					OutputDebugStringW(pDebugBuffer);

					FileSigned = WinTrustProxySvcCheckSignature(pFileName);
	
					ProcHiderDeviceResponseArray[HANDLE_ITEM] = ((DWORD*)pDeviceOutputBuffer)[0];

					if(FileSigned) 
					{
						OutputDebugString(_T("DriverControlThread - File signature verify: OK\n"));
	
						ProcHiderDeviceResponseArray[RESPONSE_ITEM] = 1;
					}
					else
					{
						OutputDebugString(_T("DriverControlThread - File signature verify: FAILED\n"));
	
						ProcHiderDeviceResponseArray[RESPONSE_ITEM] = 0;
					}
					
					ResetEvent(OverlapResult.hEvent); //DeviceIoControl will reset the event for us
					
					//
					//It would be unnecessary to wait for data because this is an input to the driver
					//and we know the driver would not return STATUS_PENDING.
					//But for the sake of completeness I first have to see if i have wait for the driver
					//and IO manager to have finished this request and thus finished using the OVERLAPPED
					//event before continuing, even though we know we will never be in this situation
					//
					DeviceIoControlResult =
						DeviceIoControl(
							hProcHiderDevice,
							IOCTL_TESTDRIVER_HIDE_PROCESS_ACK,
							ProcHiderDeviceResponseArray,
							sizeof(DWORD) * 2, //number of bytes
							NULL,
							0, 
							NULL, //we dont need this
							&OverlapResult
							);

					LastError = GetLastError();

					if(DeviceIoControlResult == 0)
					{
						switch(LastError)
						{
						case ERROR_IO_PENDING:
						case ERROR_IO_INCOMPLETE:
							WaitResult = 
								WaitForMultipleObjects(
									2,
									ShutdownAndIOEventArray,
									FALSE,
									INFINITE
									);
							switch(WaitResult)
							{
							case SHUTDOWN_EVENT:			
								OutputDebugString(_T("DriverControlThread - Shutdown received\n"));
								
								ShutDownRequested = TRUE;
				
								CancelIo(hProcHiderDevice);
			
								break;
							case IO_EVENT:			
								OutputDebugString(_T("DriverControlThread - Completed ACK\n"));
			
								break;
							default:			
								OutputDebugString(_T("DriverControlThread - Terminated ACK wait with no reason\n"));

								UnknownError = TRUE;
			
								break;
							} //switch(WaitResult)

							break;
						default:
							UnknownError = TRUE;

							break;
						} //switch(LastError)
					} //if(DeviceIoControlResult == 0)


				} //if(!UnknownError && GotData)
			}

			if(UnknownError)
			{
				if(hProcHiderDevice != INVALID_HANDLE_VALUE)
				{
					CancelIo(hProcHiderDevice);

					CloseHandle(hProcHiderDevice);
				}

				hProcHiderDevice = INVALID_HANDLE_VALUE;
				
				if(OverlapResult.hEvent != NULL)				
					CloseHandle(OverlapResult.hEvent);

				OverlapResult.hEvent = NULL;
			} //if(UnknownError)

		} //while(hProcHiderDevice != INVALID_HANDLE_VALUE && !ShutDownRequested && !UnknownError)

		if(hProcHiderDevice == INVALID_HANDLE_VALUE && !UnknownError) Sleep(5000);

	} while(hProcHiderDevice == INVALID_HANDLE_VALUE && !ShutDownRequested);

	if(hProcHiderDevice != INVALID_HANDLE_VALUE) 
		CloseHandle(hProcHiderDevice);

	if(OverlapResult.hEvent != NULL)
		CloseHandle(OverlapResult.hEvent);

	free(pDebugBuffer);

	free(pDeviceOutputBuffer);

	return(status);
/*
	//
	//we open the device.
	//we loop in case the service is started before the driver, 
	//so we wait for Createfile to return something
	//
	do {
		hProcHiderDevice = CreateFile(
			DEVICE_NAME,
			FILE_ALL_ACCESS,
			FILE_SHARE_READ | FILE_SHARE_READ,
			NULL,
			OPEN_EXISTING,
			FILE_FLAG_OVERLAPPED, //we use Asynchronous I/O
			NULL
			);

		if(hProcHiderDevice == INVALID_HANDLE_VALUE) Sleep(5000);

	} while(hProcHiderDevice == INVALID_HANDLE_VALUE && WaitForSingleObject((HANDLE)lpParameter,0) != WAIT_OBJECT_0 );

	if(WaitForSingleObject((HANDLE)lpParameter,0) != WAIT_OBJECT_0)
	{
		while(!ShutDownRequested)
		{
			//ResetEvent(OverlapResult.hEvent);

			DeviceIoControlResult = 
				DeviceIoControl(
					hProcHiderDevice,
					IOCTL_TESTDRIVER_PENDING_USERMODE_REQUEST,
					NULL,
					0,
					pDeviceOutputBuffer,
					sizeof(TCHAR)*1024, //number of bytes
					&DeviceOutputCount,
					&OverlapResult
					);

			LastError = GetLastError();

			StringCchPrintf(
				pDebugBuffer,
				1024,
				_T("DriverControlThread - DeviceIoControl returned: %d\n"),
				DeviceIoControlResult
				); 
	
			OutputDebugString(pDebugBuffer);
			//
			//even if we know this will be async because we know how our driver work
			//for the sake of completeness lets handle the IO in progress or IO finished
			//cases
			//
			if(DeviceIoControlResult == 0)
			{
				if(LastError == ERROR_IO_PENDING || LastError == ERROR_IO_INCOMPLETE)
				{
					//
					//we wait for I/O to complete or shutdown to be signaled
					//
					WaitResult = WaitForMultipleObjects(
						2,
						ShutdownAndIOEventArray,
						FALSE,
						INFINITE
						);
					switch(WaitResult)
					{
					case SHUTDOWN_EVENT:
						//we were signaled to shutdown so cancel outstanding I/O
						OutputDebugString(_T("DriverControlThread - Shutdown received\n"));
							
						ShutDownRequested = TRUE;
			
						CancelIo(hProcHiderDevice);
						//
						//This GetOverlappedResult should return ERROR_OPERATION_ABORTED
						//Should prevent this thread to cancel the Event while the OVERLAPPED 
						//operation is still in the I/O Manager by waiting on it
						//
						StringCchPrintf(
							pDebugBuffer,
							1024,
							_T("DriverControlThread - (SHUTDOWN_EVENT) GetOVerlappedResult returned: %d\n"),
							GetOverlappedResult(
								hProcHiderDevice,
								&OverlapResult,
								&DeviceOutputCount,
								TRUE
								)
							);
	
						OutputDebugString(pDebugBuffer);
					
						StringCchPrintf(
							pDebugBuffer,
							1024,
							_T("DriverControlThread - GetLastError (GetOverlappedResult) returned: %d\n"),
							GetLastError()
							);
		
						OutputDebugString(pDebugBuffer);
		
						break;
					case IO_EVENT:
						//
						//we were signaled that I/O completed	
						//
						GetOverlappedResult(
							hProcHiderDevice,
							&OverlapResult,
							&DeviceOutputCount,
							TRUE
							);
					
						StringCchPrintf(
							pDebugBuffer,
							1024,
							_T("DriverControlThread - I/O completed\n\t%d bytes received\n"),
							DeviceOutputCount
							);

						OutputDebugString(pDebugBuffer);
	
						pFileName = (PWCHAR)((PCHAR)pDeviceOutputBuffer + sizeof(DWORD)); //we offset to get filename
	
						if(DeviceOutputCount > 0)
						{
							StringCchPrintf(
								pDebugBuffer,
								1024,
								_T("DriverControlThread - (IO_EVENT) GetOVerlappedResult returned %d bytes\n\tHandle: %d\n"),
								DeviceOutputCount,
								(DWORD*)pDeviceOutputBuffer[0]
								);
							
							OutputDebugString(pDebugBuffer);
							//
							//we strictly use a wide char debug string because the name from driver is 
							//returned as UNICODE to append the filename for debug
							//we could have used a strictly ANSI one and used %S instead of %s and the ANSI
							//version of OutputDebugString
							//
							StringCbPrintfW(
								pDebugBuffer,
								1024 * sizeof(TCHAR), //to have no problem if i am using ANSI or UNICODE i use a byte counted function instead of a char counted
								L"\tFilename: %s\n",
								pFileName
								);
	
							OutputDebugStringW(pDebugBuffer);
							
							//this was for debugging purposes
							//printing the buffer in hex
							for(i=0; i<DeviceOutputCount; i++)
							{
								StringCchPrintf(
									pDebugBuffer,
									1024,
									_T("%0.2X"),
									((PUCHAR)pDeviceOutputBuffer)[i]
									);
	
								OutputDebugString(pDebugBuffer);
							}
	
							OutputDebugString(_T("\n"));
							
							FileSigned = WinTrustProxySvcCheckSignature(pFileName);
	
							ProcHiderDeviceResponseArray[HANDLE_ITEM] = ((DWORD*)pDeviceOutputBuffer)[0];
	
							if(FileSigned) 
							{
								OutputDebugString(_T("DriverControlThread - File signature verify: OK\n"));
	
								ProcHiderDeviceResponseArray[RESPONSE_ITEM] = 1;
							}
							else
							{
								OutputDebugString(_T("DriverControlThread - File signature verify: FAILED\n"));
	
								ProcHiderDeviceResponseArray[RESPONSE_ITEM] = 0;
							}
					
							ResetEvent(OverlapResult.hEvent); //DeviceIoControl will reset the event for us
					
							//
							//It would be unnecessary to wait for data because this is an input to the driver
							//and we know the driver would not return STATUS_PENDING.
							//But for the sake of completeness I first have to see if i have wait for the driver
							//and IO manager to have finished this request and thus finished using the OVERLAPPED
							//event before continuing, even though we know we will never be in this situation
							//
							if(DeviceIoControl(
								hProcHiderDevice,
								IOCTL_TESTDRIVER_HIDE_PROCESS_ACK,
								ProcHiderDeviceResponseArray,
								sizeof(DWORD) * 2, //number of bytes
								NULL,
								0, 
								NULL, //we dont need this
								&OverlapResult
								) == 0
							)
							{
								LastError = GetLastError();

								OutputDebugString(_T("DriverControlThread - ACK returned pending\n"));
	
								if(LastError == ERROR_IO_PENDING || LastError == ERROR_IO_INCOMPLETE)
								{
									switch(
										WaitForMultipleObjects(
											2,
											ShutdownAndIOEventArray,
											FALSE,
											INFINITE
											)
										)
									{
									case SHUTDOWN_EVENT:
			
										OutputDebugString(_T("DriverControlThread - Shutdown received\n"));
								
										ShutDownRequested = TRUE;
				
										CancelIo(hProcHiderDevice);
			
										break;
									case IO_EVENT:
			
										OutputDebugString(_T("DriverControlThread - Completed ACK\n"));
			
										break;
									default:
			
										OutputDebugString(_T("DriverControlThread - Terminated ACK wait with no reason\n"));
			
										break;
									}
								}
							}
						}
						break;
					default:
						break;
					} //switch(WaitResult)
				} //if(LastError == ERROR_IO_PENDING || LastError == ERROR_IO_INCOMPLETE)
			} //if(!DeviceIoControlResult)
		} //while(!ShutDownRequested)

		CloseHandle(hProcHiderDevice);
	} //if(WaitForSingleObject((HANDLE)lpParameter,0) != WAIT_OBJECT_0)
	else
	{
		ShutDownRequested = TRUE;

		CloseHandle(hProcHiderDevice);
	}

	if(ShutDownRequested) OutputDebugString(_T("DriverControlThread - Shutdown requested\n"));

	OutputDebugString(_T("DriverControlThread - Exiting Driver Control Thread\n"));

	free(pDebugBuffer);

	free(pDeviceOutputBuffer);

	CloseHandle(OverlapResult.hEvent);

	return(status);
*/
}