#include "WinTrustProxySvcMain.h"

TCHAR * pFullExePath = NULL;

int 
__cdecl 
_tmain(
	int argc, 
	TCHAR *argv[]
	) 
{
	GetFileNameWithPath(&pFullExePath);

	_tprintf(_T("%s\n"),pFullExePath);

	if(argc <= 1)
	{
		PrintUsage();

		return(0);
	}

	if(_tcsncmp(&argv[1][0],_T("-"),1) != 0)
	{
		PrintUsage();

		return(0);
	}

	if(_tcsncmp(&argv[1][1],_T("i"),1) == 0)
	{	
		_tprintf(MSG_INFO_INSTALL_REQUEST);

		DecodeInstallError(InstallService(pFullExePath));
		/*
		if (InstallService(pFullExePath) != INSTALL_SVC_NO_ERROR)
		{
			_tprintf(MSG_ERR_INSTALL_FAIL);
		}
		else _tprintf(MSG_INFO_INSTALL_SUCCESFUL);
		*/
	}
	else if(_tcsncmp(&argv[1][1],_T("u"),1) == 0)
	{
		_tprintf(MSG_INFO_UNINSTALL_REQUEST);

		DecodeUninstallError(UninstallService());
		/*
		if (UninstallService() != INSTALL_SVC_NO_ERROR)
		{
			_tprintf(MSG_ERR_UNINSTALL_FAIL);
		}
		else _tprintf(MSG_INFO_UNINSTALL_SUCCESFUL);
		*/
	}
	else if(_tcsncmp(&argv[1][1],_T("s"),1) == 0)
	{
		_tprintf(MSG_INFO_SERVICE_START_REQUEST);

		if (lStartService() != START_SVC_NO_ERROR)
		{
			_tprintf(MSG_ERR_SERVICE_START_FAIL);
		}
		else _tprintf(MSG_INFO_SERVICE_START_SUCCESFUL);
	}
	else if(_tcsncmp(&argv[1][1],_T("k"),1) == 0)
	{
		_tprintf(MSG_INFO_SERVICE_STOP_REQUEST);

		if(StopService() != START_SVC_NO_ERROR)
		{
			_tprintf(MSG_ERR_SERVICE_STOP_FAIL);
		}
		else _tprintf(MSG_INFO_SERVICE_STOP_SUCCESFUL);
	}
	else if(_tcsncmp(&argv[1][1],_T("r"),1) == 0)
	{
		RunService();		
	}
	else
	{
		_tprintf(MSG_ERR_UNKNOWN_SWITCH);

		PrintUsage();
	}

	return(0);
};

void 
PrintUsage(
	void
	)
{
	_tprintf(MSG_USAGE_MSG);
};


RUN_SVC_ERROR
RunService(
	void
	)
{
	RUN_SVC_ERROR status = RUN_SVC_NO_ERROR;

	if(!StartServiceCtrlDispatcher(DispatchTable))
	{
		switch(GetLastError())
		{
		case ERROR_FAILED_SERVICE_CONTROLLER_CONNECT:
			status = RUN_SVC_SERVICE_CONTROLLER_CONNECT_FAILED;
			break;
		case ERROR_INVALID_DATA:
			status = RUN_SVC_DISPATCH_TABLE_INVALID;
			break;
		case ERROR_SERVICE_ALREADY_RUNNING:
			status = RUN_SVC_SERVICE_ALREADY_RUNNING;
			break;
		default:
			status = RUN_SVC_UNKNOWN_ERROR;
			break;
		}
	}

	return(status);
}


START_SVC_ERROR
StopService(
	void
	)
{
	SC_HANDLE hServiceManager = NULL;
	SC_HANDLE hService = NULL;
	START_SVC_ERROR status = START_SVC_NO_ERROR;
	SERVICE_STATUS ServiceStatus = {0};

	hServiceManager = OpenSCManager(NULL,NULL,START_SVC_MANAGER_FLAG);

	if(hServiceManager == NULL)
	{
		status = START_SVC_CANNOT_OPEN_MANAGER;

		goto exit_err;
	}

	hService = OpenService(
		hServiceManager,
		SERVICE_NAME,
		SERVICE_STOP | SERVICE_QUERY_STATUS
		);

	if(hService == NULL)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = START_SVC_CANNOT_OPEN_SVC_FOR_STOP;
			break;
		case ERROR_INVALID_HANDLE:
			status = START_SVC_INVALID_MANAGER_HANDLE;
			break;
		case ERROR_INVALID_NAME:
			status = START_SVC_INVALID_SERVICE_NAME;
			break;
		case ERROR_SERVICE_DOES_NOT_EXIST:
			status = START_SVC_SERVICE_DONT_EXIST;
			break;
		default:
			status = START_SVC_CANNOT_OPEN_SVC_FOR_STOP;
			break;
		}

		goto exit_err;
	}

	if(ControlService(hService,SERVICE_CONTROL_STOP,&ServiceStatus) == 0)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = START_SVC_CANNOT_STOP_SERVICE;
			break;
		case ERROR_DEPENDENT_SERVICES_RUNNING:
			status = START_SVC_DEPENDENT_SERVICES_RUNNING;
			break;
		case ERROR_INVALID_HANDLE:
			status = START_SVC_INVALID_SERVICE_HANDLE;
			break;
		case ERROR_INVALID_PARAMETER:
			status = START_SVC_UNDEFINED_CODE;
			break;
		case ERROR_INVALID_SERVICE_CONTROL:
			status = START_SVC_INVALID_SERVICE_CONTROL;
			break;
		case ERROR_SERVICE_CANNOT_ACCEPT_CTRL:
			status = START_SVC_CANNOT_ACCEPT_CTRL;
			break;
		case ERROR_SERVICE_NOT_ACTIVE:
			status = START_SVC_SERVICE_NOT_ACTIVE;
			break;
		case ERROR_SERVICE_REQUEST_TIMEOUT:
			status = START_SVC_REQUEST_TIMEOUT;
			break;
		case ERROR_SHUTDOWN_IN_PROGRESS:
			status = START_SVC_SHUTDOWN_IN_PROGRESS;
			break;
		default:
			status = START_SVC_CANNOT_STOP_SERVICE;
			break;
		}
	}

exit_err:

	if(hServiceManager != NULL) CloseServiceHandle(hServiceManager);

	if(hService != NULL) CloseServiceHandle(hService);

	return(status);
}

void
DecodeInstallError(
	INSTALL_SVC_ERROR ErrorCode
	)
{
	switch(ErrorCode)
	{
	case INSTALL_SVC_NO_ERROR:
		_tprintf(MSG_INFO_INSTALL_SUCCESFUL);
		break;
	case INSTALL_SVC_SERVICE_ALREADY_EXISTS:
		_tprintf(MSG_ERR_SERVICE_ALREADY_EXISTS);
		break;
	case INSTALL_SVC_SERVICE_ALREADY_MARKED_FOR_DELETION:
		_tprintf(MSG_ERR_SERVICE_ALREADY_MARKED_FOR_DELETION);
		break;
	case INSTALL_SVC_CANNOT_CREATE_SERVICE:
		_tprintf(MSG_ERR_CANNOT_CREATE_SERVICE);
		break;
	case INSTALL_SVC_CANNOT_OPEN_MANAGER:
		_tprintf(MSG_ERR_CANNOT_OPEN_MANAGER);
		break;
	default:
		_tprintf(MSG_ERR_INSTALL_FAIL);
		_tprintf(_T("Error Code: %d\n"), ErrorCode);
		break;
	}
}

INSTALL_SVC_ERROR 
InstallService(
	TCHAR * pFilePathName
	)
{
	SC_HANDLE hServiceManager = NULL;
	SC_HANDLE hService = NULL;
	INSTALL_SVC_ERROR status = INSTALL_SVC_NO_ERROR;
	TCHAR * pServiceCommandLine = NULL;
	TCHAR * pServiceCommandLineQuoted = NULL;
	size_t ServiceCommandLineSize = 0;
	size_t ServiceCommandLineQuotedSize = 0;

	if(QuoteString(pFilePathName,&pServiceCommandLineQuoted,&ServiceCommandLineQuotedSize) != STRING_HELP_NO_ERROR)
	{
		status = INSTALL_SVC_CANNOT_BUILD_SVC_CMD_LINE;

		goto exit_err;
	}

	if(AllocateStringSpaceForCatSafe(pServiceCommandLineQuoted,SERVICE_START_ARG,&pServiceCommandLine, &ServiceCommandLineSize) != STRING_HELP_NO_ERROR)
	{
		status = INSTALL_SVC_CANNOT_ALLOC_RESOURCES;

		goto exit_err;
	}

	if(StringCat(pServiceCommandLineQuoted,SERVICE_START_ARG,pServiceCommandLine,ServiceCommandLineSize) != STRING_HELP_NO_ERROR)
	{
		status = INSTALL_SVC_CANNOT_BUILD_SVC_CMD_LINE;

		goto exit_err;
	}	
	//
	//we get the handle to the service manager
	//
	hServiceManager = OpenSCManager(NULL,NULL,INSTALL_SVC_MANAGER_FLAG);

	if(hServiceManager == NULL)
	{
		status = INSTALL_SVC_CANNOT_OPEN_MANAGER;

		goto exit_err;
	}

	hService = CreateService(
		hServiceManager,
		SERVICE_NAME,
		SERVICE_DISPLAY_NAME,
		INSTALL_SVC_SERVICE_FLAG,
		SERVICE_WIN32_OWN_PROCESS,
		SERVICE_DEMAND_START,
		SERVICE_ERROR_NORMAL,
		pServiceCommandLine,
		NULL,
		NULL,
		NULL,
		NULL,
		NULL
		);

	if(hService == NULL)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = INSTALL_SVC_CANNOT_CREATE_SERVICE;
			break;
		case ERROR_CIRCULAR_DEPENDENCY:
			status = INSTALL_SVC_CIRCULAR_DEPENDENCY;
			break;
		case ERROR_DUPLICATE_SERVICE_NAME:
			status = INSTALL_SVC_DUPLICATE_NAME;
			break;
		case ERROR_INVALID_HANDLE:
			status = INSTALL_SVC_INVALID_MANAGER_HANDLE;
			break;
		case ERROR_INVALID_NAME:
			status = INSTALL_SVC_INVALID_SERVICE_NAME;
			break;
		case ERROR_INVALID_PARAMETER:
			status = INSTALL_SVC_INVALID_PARAMETER;
			break;
		case ERROR_INVALID_SERVICE_ACCOUNT:
			status = INSTALL_SVC_INVALID_SERVICE_ACCOUNT;
			break;
		case ERROR_SERVICE_EXISTS:
			status = INSTALL_SVC_SERVICE_ALREADY_EXISTS;
			break;
		case ERROR_SERVICE_MARKED_FOR_DELETE:
			status = INSTALL_SVC_SERVICE_ALREADY_MARKED_FOR_DELETION;
			break;
		default:
			status = INSTALL_SVC_CANNOT_CREATE_SERVICE;
			break;
		}

		goto exit_err;
	}

exit_err:
	
	if(hServiceManager != NULL) CloseServiceHandle(hServiceManager);

	if(hService != NULL) CloseServiceHandle(hService);

	if(pServiceCommandLine != NULL) FreeStringSpace(pServiceCommandLine);

	if(pServiceCommandLineQuoted != NULL) FreeStringSpace(pServiceCommandLineQuoted);

	return(status);
};

void
DecodeUninstallError(
	INSTALL_SVC_ERROR ErrorCode
	)
{
	switch(ErrorCode)
	{
	case INSTALL_SVC_NO_ERROR:
		_tprintf(MSG_INFO_UNINSTALL_SUCCESFUL);
		break;
	case INSTALL_SVC_CANNOT_DELETE_SERVICE:
		_tprintf(MSG_ERR_CANNOT_DELETE_SERVICE);
		break;
	case INSTALL_SVC_CANNOT_OPEN_MANAGER:
		_tprintf(MSG_ERR_CANNOT_OPEN_MANAGER);
		break;
	case INSTALL_SVC_CANNOT_OPEN_SVC_FOR_DELETE:
		_tprintf(MSG_ERR_CANNOT_OPEN_SVC_FOR_DELETE);
		break;
	case INSTALL_SVC_SERVICE_DONT_EXIST:
		_tprintf(MSG_ERR_SERVICE_DONT_EXIST);
		break;
	case INSTALL_SVC_SERVICE_ALREADY_MARKED_FOR_DELETION:
		_tprintf(MSG_ERR_SERVICE_ALREADY_MARKED_FOR_DELETION);
		break;
	case INSTALL_SVC_INVALID_SERVICE_HANDLE:
		_tprintf(MSG_ERR_INVALID_SERVICE_HANDLE);
		break;
	default:
		_tprintf(MSG_ERR_UNINSTALL_FAIL);
		_tprintf(_T("Error Code: %d\n"), ErrorCode);
		break;
	}
}

INSTALL_SVC_ERROR 
UninstallService(
	void
	)
{
	SC_HANDLE hServiceManager = NULL;
	SC_HANDLE hService = NULL;
	INSTALL_SVC_ERROR status = INSTALL_SVC_NO_ERROR;

	hServiceManager = OpenSCManager(NULL,NULL,INSTALL_SVC_MANAGER_FLAG);

	if(hServiceManager == NULL)
	{
		status = INSTALL_SVC_CANNOT_OPEN_MANAGER;

		goto exit_err;
	}

	hService = OpenService(
		hServiceManager,
		SERVICE_NAME,
		DELETE
		);

	if(hService == NULL)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = INSTALL_SVC_CANNOT_OPEN_SVC_FOR_DELETE;
			break;
		case ERROR_INVALID_HANDLE:
			status = INSTALL_SVC_INVALID_MANAGER_HANDLE;
			break;
		case ERROR_INVALID_NAME:
			status = INSTALL_SVC_INVALID_SERVICE_NAME;
			break;
		case ERROR_SERVICE_DOES_NOT_EXIST:
			status = INSTALL_SVC_SERVICE_DONT_EXIST;
			break;
		default:
			status = INSTALL_SVC_CANNOT_OPEN_SVC_FOR_DELETE;
			break;
		}

		goto exit_err;
	}

	if(DeleteService(hService) == 0)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = INSTALL_SVC_CANNOT_DELETE_SERVICE;
			break;
		case ERROR_INVALID_HANDLE:
			status = INSTALL_SVC_INVALID_SERVICE_HANDLE;
			break;
		case ERROR_SERVICE_MARKED_FOR_DELETE:
			status = INSTALL_SVC_SERVICE_ALREADY_MARKED_FOR_DELETION;
			break;
		default:
			status = INSTALL_SVC_CANNOT_DELETE_SERVICE;
			break;
		}

		goto exit_err;
	}

exit_err:

	if(hServiceManager != NULL) CloseServiceHandle(hServiceManager);

	if(hService != NULL) CloseServiceHandle(hService);

	return(status);
}

START_SVC_ERROR
lStartService(
	void
	)
{
	SC_HANDLE hServiceManager = NULL;
	SC_HANDLE hService = NULL;
	START_SVC_ERROR status = START_SVC_NO_ERROR;

	hServiceManager = OpenSCManager(NULL,NULL,START_SVC_MANAGER_FLAG);

	if(hServiceManager == NULL)
	{
		status = START_SVC_CANNOT_OPEN_MANAGER;

		goto exit_err;
	}

	hService = OpenService(
		hServiceManager,
		SERVICE_NAME,
		SERVICE_START
		);

	if(hService == NULL)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = START_SVC_CANNOT_OPEN_SVC_FOR_START;
			break;
		case ERROR_INVALID_HANDLE:
			status = START_SVC_INVALID_MANAGER_HANDLE;
			break;
		case ERROR_INVALID_NAME:
			status = START_SVC_INVALID_SERVICE_NAME;
			break;
		case ERROR_SERVICE_DOES_NOT_EXIST:
			status = START_SVC_SERVICE_DONT_EXIST;
			break;
		default:
			status = START_SVC_CANNOT_OPEN_SVC_FOR_START;
			break;
		}

		goto exit_err;
	}

	if(StartService(hService,0,NULL) == 0)
	{
		switch(GetLastError())
		{
		case ERROR_ACCESS_DENIED:
			status = START_SVC_CANNOT_START_SERVICE;
			break;
		case ERROR_INVALID_HANDLE:
			status = START_SVC_INVALID_SERVICE_HANDLE;
			break;
		case ERROR_PATH_NOT_FOUND:
			status = START_SVC_SERVICE_PATH_NOT_FOUND;
			break;
		case ERROR_FILE_NOT_FOUND:
			status = START_SVC_SERVICE_FILE_NOT_FOUND;
			break;
		case ERROR_SERVICE_ALREADY_RUNNING:
			status = START_SVC_SERVICE_ALREADY_RUNNING;
			break;
		case ERROR_SERVICE_DATABASE_LOCKED:
			status = START_SVC_DATABASE_IS_LOCKED;
			break;
		case ERROR_SERVICE_DEPENDENCY_DELETED:
			status = START_SVC_DEPENDENCY_DELETED;
			break;
		case ERROR_SERVICE_DEPENDENCY_FAIL:
			status = START_SVC_DEPENDENCY_FAILED;
			break;
		case ERROR_SERVICE_DISABLED:
			status = START_SVC_SERVICE_DISABLED;
			break;
		case ERROR_SERVICE_LOGON_FAILED:
			status = START_SVC_LOGON_FAILED;
			break;
		case ERROR_SERVICE_MARKED_FOR_DELETE:
			status = START_SVC_SERVICE_MARKED_FOR_DELETION;
			break;
		case ERROR_SERVICE_NO_THREAD:
			status = START_SVC_CANNOT_CREATE_THREAD;
			break;
		case ERROR_SERVICE_REQUEST_TIMEOUT:
			status = START_SVC_REQUEST_TIMEOUT;
			break;
		default:
			status = START_SVC_CANNOT_START_SERVICE;
			break;
		}

		goto exit_err;
	}

exit_err:

	if(hServiceManager != NULL) CloseServiceHandle(hServiceManager);

	if(hService != NULL) CloseServiceHandle(hService);

	return(status);
}


void
GetFileNameWithPath(
	TCHAR ** ppDestination
	)
{
	_get_tpgmptr(ppDestination);
}
