#ifndef WINTRUSTPROXYSVC_MAIN_H
#define WINTRUSTPROXYSVC_MAIN_H

#include "WinTrustProxySvcCommonWinHeader.h"
#include "WinTrustProxySvcCommonHeader.h"
#include "WinTrustProxySvcStringHelp.h"
#include "WinTrustProxySvc.h"

#define INSTALL_SVC_MANAGER_FLAG SC_MANAGER_CREATE_SERVICE | SC_MANAGER_CONNECT
#define INSTALL_SVC_SERVICE_FLAG STANDARD_RIGHTS_EXECUTE | SC_MANAGER_CONNECT
#define START_SVC_MANAGER_FLAG SC_MANAGER_CONNECT | STANDARD_RIGHTS_REQUIRED

enum RUN_SVC_ERROR
{
	RUN_SVC_NO_ERROR,
	RUN_SVC_SERVICE_CONTROLLER_CONNECT_FAILED,
	RUN_SVC_DISPATCH_TABLE_INVALID,
	RUN_SVC_SERVICE_ALREADY_RUNNING,
	RUN_SVC_UNKNOWN_ERROR
};

enum INSTALL_SVC_ERROR
{
	INSTALL_SVC_NO_ERROR,
	INSTALL_SVC_INVALID_PARAMETER,
	INSTALL_SVC_INVALID_SERVICE_ACCOUNT,
	INSTALL_SVC_CANNOT_OPEN_MANAGER,
	INSTALL_SVC_CANNOT_ALLOC_RESOURCES,
	INSTALL_SVC_CANNOT_BUILD_SVC_CMD_LINE,
	INSTALL_SVC_CANNOT_CREATE_SERVICE,
	INSTALL_SVC_CANNOT_OPEN_SVC_FOR_DELETE,
	INSTALL_SVC_CANNOT_DELETE_SERVICE,
	INSTALL_SVC_INVALID_MANAGER_HANDLE,
	INSTALL_SVC_INVALID_SERVICE_NAME,
	INSTALL_SVC_INVALID_SERVICE_HANDLE,
	INSTALL_SVC_SERVICE_DONT_EXIST,
	INSTALL_SVC_SERVICE_ALREADY_MARKED_FOR_DELETION,
	INSTALL_SVC_SERVICE_ALREADY_EXISTS,
	INSTALL_SVC_CIRCULAR_DEPENDENCY,
	INSTALL_SVC_DUPLICATE_NAME
};

enum START_SVC_ERROR
{
	START_SVC_NO_ERROR,
	START_SVC_CANNOT_OPEN_MANAGER,
	START_SVC_CANNOT_OPEN_SVC_FOR_START,
	START_SVC_CANNOT_OPEN_SVC_FOR_STOP,
	START_SVC_INVALID_MANAGER_HANDLE,
	START_SVC_INVALID_SERVICE_NAME,
	START_SVC_SERVICE_DONT_EXIST,
	START_SVC_CANNOT_START_SERVICE,
	START_SVC_CANNOT_STOP_SERVICE,
	START_SVC_INVALID_SERVICE_HANDLE,
	START_SVC_UNDEFINED_CODE,
	START_SVC_INVALID_SERVICE_CONTROL,
	START_SVC_CANNOT_ACCEPT_CTRL,
	START_SVC_SERVICE_PATH_NOT_FOUND,
	START_SVC_SERVICE_FILE_NOT_FOUND,
	START_SVC_SERVICE_ALREADY_RUNNING,
	START_SVC_DATABASE_IS_LOCKED,
	START_SVC_DEPENDENCY_DELETED,
	START_SVC_DEPENDENCY_FAILED,
	START_SVC_DEPENDENT_SERVICES_RUNNING,
	START_SVC_SERVICE_DISABLED,
	START_SVC_SERVICE_NOT_ACTIVE,
	START_SVC_LOGON_FAILED,
	START_SVC_SERVICE_MARKED_FOR_DELETION,
	START_SVC_CANNOT_CREATE_THREAD,
	START_SVC_REQUEST_TIMEOUT,
	START_SVC_SHUTDOWN_IN_PROGRESS
};

TCHAR SERVICE_NAME[] = _T("WinTrustProxySvc");

TCHAR SERVICE_DISPLAY_NAME[] = _T("WinTrustProxy Service");

TCHAR DEVICE_NAME[] = _T("\\\\.\\ProcHider");

TCHAR SERVICE_START_ARG[] = _T(" -r");

TCHAR MSG_USAGE_MSG[] = _T("Usage: WinTrustProxySvc.exe {-i|-k|-s|-u}\n\n\t-i\tInstall the service\n\t-k\tKill the service\n\t-s\tStart the service\n\t-u\tUninstall the service\n\nX-Treem for quequero.org\n");

TCHAR MSG_ERR_UNKNOWN_SWITCH[] = _T("Error: Unknow switch\n");

TCHAR MSG_ERR_INSTALL_FAIL[] = _T("Error: Service installation failed\n");

TCHAR MSG_ERR_UNINSTALL_FAIL[] = _T("Error: Service uninstallation failed\n");

TCHAR MSG_ERR_SERVICE_START_FAIL[] = _T("Error: Service start failed\n");

TCHAR MSG_ERR_SERVICE_STOP_FAIL[] = _T("Error: Service stop failed\n");


TCHAR MSG_ERR_SERVICE_ALREADY_EXISTS[] = _T("Error: Service already exists\n");

TCHAR MSG_ERR_SERVICE_ALREADY_MARKED_FOR_DELETION[] = _T("Error: Service marked for deletion\n");

TCHAR MSG_ERR_CANNOT_CREATE_SERVICE[] = _T("Error: Cannot create service\n");

TCHAR MSG_ERR_CANNOT_OPEN_MANAGER[] = _T("Error: Cannot open Service Control manager\n");

TCHAR MSG_ERR_CANNOT_DELETE_SERVICE[] = _T("Error: Cannot delete service\n");

TCHAR MSG_ERR_CANNOT_OPEN_SVC_FOR_DELETE[] = _T("Error: Cannot open service for delete\n");

TCHAR MSG_ERR_SERVICE_DONT_EXIST[] = _T("Error: Service does not exist\n");

TCHAR MSG_ERR_INVALID_SERVICE_HANDLE[] = _T("Error: Invalid service handle\n");


TCHAR MSG_INFO_START_REQUEST[] = _T("Info: Requested service start\n");

TCHAR MSG_INFO_INSTALL_REQUEST[] = _T("Info: Requested service install\n");

TCHAR MSG_INFO_UNINSTALL_REQUEST[] = _T("Info: Requested service uninstall\n");

TCHAR MSG_INFO_INSTALL_SUCCESFUL[] = _T("Info: Service install succesful\n");

TCHAR MSG_INFO_UNINSTALL_SUCCESFUL[] = _T("Info: Service uninstall succesful\n");

TCHAR MSG_INFO_SERVICE_START_REQUEST[] = _T("Info: Requested service start\n");

TCHAR MSG_INFO_SERVICE_START_SUCCESFUL[] = _T("Info: Service start succesful\n");

TCHAR MSG_INFO_SERVICE_STOP_REQUEST[] = _T("Info: Requested service stop\n");

TCHAR MSG_INFO_SERVICE_STOP_SUCCESFUL[] = _T("Info: Service stop succesful\n");

void 
PrintUsage(
	void
	);

INSTALL_SVC_ERROR 
InstallService(
	TCHAR * pFilePathName
	);

void
DecodeInstallError(
	INSTALL_SVC_ERROR ErrorCode
	);

INSTALL_SVC_ERROR 
UninstallService(
	void
	);

void
DecodeUninstallError(
	INSTALL_SVC_ERROR ErrorCode
	);

START_SVC_ERROR
lStartService(
	void
	);

START_SVC_ERROR
StopService(
	void
	);

RUN_SVC_ERROR
RunService(
	void
	);

void 
GetFileNameWithPath(
	TCHAR ** ppDestination
	);


SERVICE_TABLE_ENTRY DispatchTable[] = 
    { 
        { SERVICE_NAME, (LPSERVICE_MAIN_FUNCTION) WinTrustProxySvc }, 
        { NULL, NULL } 
    };

#endif //WINTRUSTPROXYSVC_MAIN_H