#include "WinTrustProxySvcStringHelp.h"

TCHAR * pQuote = _T("\""); //local to the module

STRING_HELP
AllocateStringSpace(
	TCHAR ** ppSpace,
	size_t SpaceSize
	)
{
	STRING_HELP result = STRING_HELP_NO_ERROR;

	if(ppSpace == NULL || SpaceSize <= 0) result = STRING_HELP_INVALID_PARAMETER;
	else
	{
		__try {

			*ppSpace = (TCHAR*)malloc(SpaceSize);

		} __except(StringHelpExceptionFilter(GetExceptionCode(), GetExceptionInformation())) {

			if(*ppSpace != NULL) free(*ppSpace);

			*ppSpace = NULL;

			result = STRING_HELP_INVALID_PARAMETER;

		}
	}

	return(result);
}

STRING_HELP 
AllocateStringSpaceForCatSafe(
	TCHAR * pString1, 
	TCHAR * pString2, 
	TCHAR ** ppDestination, //output
	size_t * pDestSize //output
	)
{
	size_t String1Length = 0;
	size_t String2Length = 0;
	size_t DestSize = 0;

	STRING_HELP result = STRING_HELP_NO_ERROR;

	if(pString1 == NULL || pString2 == NULL || pDestSize == NULL) result = STRING_HELP_INVALID_PARAMETER;
	else
	{
		__try {

			if(StringCbLength(pString1,STRSAFE_MAX_CCH,&String1Length) != S_OK) 
			{
				result = STRING_HELP_INVALID_PARAMETER; //null char not counted

				RaiseException(
					EXCEPTION_ACCESS_VIOLATION,
					0,
					0,
					NULL
					);
			}

			if(StringCbLength(pString2,STRSAFE_MAX_CCH,&String2Length) != S_OK)
			{
				result =STRING_HELP_INVALID_PARAMETER; //null char not counted

				RaiseException(
					EXCEPTION_ACCESS_VIOLATION,
					0,
					0,
					NULL
					);
			}

			DestSize = String1Length + String2Length + (sizeof(TCHAR)); //we check overflow against size_t (handled in exception filter)

			*ppDestination = (TCHAR*)malloc(DestSize);

			if(*ppDestination==NULL) result = STRING_HELP_CANNOT_ALLOC_RESOURCES;

			if(*ppDestination!=NULL) ZeroMemory(*ppDestination,DestSize);

			*pDestSize = DestSize;

		} __except(StringHelpExceptionFilter(GetExceptionCode(), GetExceptionInformation())) {

			if(*ppDestination != NULL) free(*ppDestination);

			*ppDestination = NULL;

			result = STRING_HELP_INVALID_PARAMETER;
			
			if(GetExceptionCode() == EXCEPTION_INT_OVERFLOW) result = STRING_HELP_STRING_TOO_BIG;
		}
	}

	return(result);
}

STRING_HELP 
StringCat(
	TCHAR * pString1,
	TCHAR * pString2, //optional
	TCHAR * pDestination,
	size_t DestSize
	)
{
	STRING_HELP result = STRING_HELP_NO_ERROR;

	if(pString1 == NULL || pDestination == NULL || DestSize <= 0) result = STRING_HELP_INVALID_PARAMETER;
	else
	{
		__try {

			switch(StringCbCat(pDestination,DestSize,pString1))
			{
			case S_OK:
				break;
			case STRSAFE_E_INVALID_PARAMETER:
				result = STRING_HELP_INVALID_PARAMETER;
				__leave;
				break;
			case STRSAFE_E_INSUFFICIENT_BUFFER:
				result = STRING_HELP_BUFFER_TOO_SMALL;
				__leave;
				break;
			default:
				result = STRING_HELP_UNKNOWN_ERROR;
				__leave;
				break;
			};
			
			if(pString2 != NULL)
				switch(StringCbCat(pDestination,DestSize,pString2))
				{
				case S_OK:
					break;
				case STRSAFE_E_INVALID_PARAMETER:
					result = STRING_HELP_INVALID_PARAMETER;
					__leave;
					break;
				case STRSAFE_E_INSUFFICIENT_BUFFER:
					result = STRING_HELP_BUFFER_TOO_SMALL;
					__leave;
					break;
				default:
					result = STRING_HELP_UNKNOWN_ERROR;
					__leave;
					break;
				}			

		} __except(StringHelpExceptionFilter(GetExceptionCode(), GetExceptionInformation())) {

			result = STRING_HELP_INVALID_PARAMETER;
		}
	}

	return(result);
}


STRING_HELP
QuoteString(
	TCHAR * pSource,
	TCHAR ** ppDestination,
	size_t * pDestSize
	)
{
	size_t SourceSize = 0;
	size_t QuoteSize = 0;
	size_t DestSize = 0;
	STRING_HELP result = STRING_HELP_NO_ERROR;

	if(pSource == NULL || ppDestination == NULL || pDestSize == NULL) result = STRING_HELP_INVALID_PARAMETER;
	else
	{
		__try {

			if(StringCbLength(pQuote,STRSAFE_MAX_CCH,&QuoteSize) != S_OK)
			{
				result = STRING_HELP_INVALID_PARAMETER; //null char not counted

				RaiseException(
					EXCEPTION_ACCESS_VIOLATION,
					0,
					0,
					NULL
					);
			}

			if(StringCbLength(pSource,STRSAFE_MAX_CCH,&SourceSize) != S_OK)
			{
				result = STRING_HELP_INVALID_PARAMETER; //null char not counted

				RaiseException(
					EXCEPTION_ACCESS_VIOLATION,
					0,
					0,
					NULL
					);
			}

			DestSize = SourceSize + (2 * QuoteSize) + sizeof(TCHAR); //account beginning quote, end quote and null terminator

			*ppDestination = (TCHAR*)malloc(DestSize);

			if(*ppDestination==NULL) result = STRING_HELP_CANNOT_ALLOC_RESOURCES;

			if(*ppDestination!=NULL) 
			{
				ZeroMemory(*ppDestination,DestSize);

				result = StringCat(pQuote,pSource,*ppDestination,DestSize);

				if(result != STRING_HELP_NO_ERROR) 
					RaiseException(
						EXCEPTION_ACCESS_VIOLATION,
						0,
						0,
						NULL
						);

				result = StringCat(pQuote,NULL,*ppDestination,DestSize);

				if(result != STRING_HELP_NO_ERROR) 
					RaiseException(
						EXCEPTION_ACCESS_VIOLATION,
						0,
						0,
						NULL
						);
			}

			*pDestSize = DestSize;

		} __except(StringHelpExceptionFilter(GetExceptionCode(), GetExceptionInformation())) {

			if(*ppDestination != NULL) free(*ppDestination);

			*ppDestination = NULL;

			result = STRING_HELP_INVALID_PARAMETER;
			
			if(GetExceptionCode() == EXCEPTION_INT_OVERFLOW) result = STRING_HELP_STRING_TOO_BIG;
		}
	}

	return(result);
}

STRING_HELP
FreeStringSpace(
	TCHAR * pString
	)
{
	STRING_HELP result = STRING_HELP_NO_ERROR;

	free(pString);

	return(result);
}

int StringHelpExceptionFilter(
	unsigned int code, 
	struct _EXCEPTION_POINTERS *ep
	)
{
	int ret = EXCEPTION_EXECUTE_HANDLER;

	if(code == EXCEPTION_ACCESS_VIOLATION || EXCEPTION_INT_OVERFLOW) ret = EXCEPTION_EXECUTE_HANDLER;
	else ret = EXCEPTION_CONTINUE_SEARCH;

	return(ret);
}